/* $Id: common-run.cpp 451 2011-02-26 10:32:30Z oh2gve $
 *
 * Copyright 2007, 2008, 2009, 2010, 2011 Tapio Aaltonen
 *
 *
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

// Own stuff.
#ifdef __WXMSW__
#	include "windows/controller.h"
#endif
#include "cfg.h"
#include "packetlog.h"
#include "dupefilter.h"
#include "beacon.h"
#include "main.h"

// Std stuff.
#include <cstdlib>
#include <vector>
using std::vector;
#include <string>
using std::string;
#ifndef __WXMSW__
#	include <csignal>
#endif

// Wx stuff.
#ifndef __WXMSW__
#	include <wx/utils.h>
#else
#	include <wx/thread.h>
#endif
#include <wx/log.h>



#ifdef __WXMSW__
wxThread::ExitCode Controller::Entry()
#else
int AprsApp::OnRun()
#endif
{
#ifndef __WXMSW__
	if ( iAmDaddy_ )
	{
		return EXIT_SUCCESS;
	}
#else
	wxLogMessage(wxT("Controller started."));
#endif

	// Begin gateway action.
	wxLogMessage(wxT("Startup successfull, entering operation mode."));
	bool noError = true;

	// Here is THE main loop.
#ifdef __WXMSW__
	while ( noError && !TestDestroy() )
#else
	while ( noError )
#endif
	{
		// Check connections for input.
		for ( unsigned int i = 0; i < connections_.size(); ++i )
		{
			// Try to read.
			Packet* packet = connections_.at(i)->read();

			// If we got sometring, process it.
			if ( packet )
			{
				// Put packet to all non-uplink connections except inputting one.
				for ( unsigned int j = 0; j < connections_.size(); j++ )
				{
					if ( i != j )
					{
/*					
						// Create gated-by-us copy of the packet and send it.
						Packet* out = new Packet(*packet);
						out->setInputName( static_cast<char const*>(configuration_.connections.at(i).name.ToAscii()) );
						connections_.at(j)->write(out);
*/					}
				}
				
				// Check if we can gate this to APRS-IS.
				if ( packet->getAprsPacket()->canBeGatedToAprsIs() )
				{
					// Create q-construct to add.
					vector<string> addPath;
					addPath.push_back("qAR");
					addPath.push_back( string(configuration_.callsign.ToAscii()) );
					
					// Create copy of the packet.
					AprsPacket* tmpPacket = new AprsPacket(*packet->getAprsPacket());
					tmpPacket->addToPath(addPath);
					packet->setAprsPacket(tmpPacket);
					
					// Then send it.
					uplink_->write(packet);
				}
				else
				{
					// Packet can't be gated, let's just log it.
					packet->setFlag('r');
					packetlog_->write(packet);
				}
			}
		}
		
		// Check uplink for input.
		while (	Packet* packet = uplink_->read() )
		{
			// If all uplink traffic is to be logged, then do it.
			if ( configuration_.logcfg.logUplink )
			{
				Packet* logpkt = new Packet(*packet);
				packetlog_->write(logpkt);
			}
		
			// Copy packet to all non-uplink connections.
			for ( unsigned int i = 0; i < connections_.size(); i++ )
			{
				Packet* out = new Packet(*packet);
				out->setInputName("TCPIP");
				connections_.at(i)->write(out);
			}
			
			delete packet;
		}

		// Check that connections are alive.
		if ( !(*uplink_) )
		{
			wxLogError(wxT("uplink has died."));
			noError = false;
			break;
		}
		for ( unsigned int i = 0; i < connections_.size(); ++i )
		{
			if ( configuration_.connections.at(i).dieOnError && !(*connections_.at(i)) )
			{
				wxLogError(wxT("connection ") + wxString::Format(wxT("%d"), i) + wxT(" has died."));
				noError = false;
				break;
			}
		}

#ifndef __WXMSW__
		// Check for signals.
		if ( gotSignal_ == SIGINT || gotSignal_ == SIGTERM )
		{
			// We need to quit.
			break;
		}
		else if ( gotSignal_ == SIGHUP )
		{
			// They want us to reopen files.
			wxLogMessage(wxT("SIGHUP received, reopening logs."));
			packetlog_->reopen();
		}
		else if ( gotSignal_ != 0 )
		{
			wxLogMessage(wxString::Format(wxT("Got signal %d from OS."), gotSignal_));
			gotSignal_ = 0;
		}
#endif
		// Check that assistants are alive.
		if ( !(*packetlog_) )
		{
			noError = false;
		}
		
		// Wait some time before next loop iteration.
#ifdef __WXMSW__
		Sleep(10);
#else
		wxMilliSleep(10);
#endif
	}
	
	// Put quitting reason in log.
	if ( noError )
	{
		wxLogMessage(wxT("Received close request from OS, shutting down."));
		// Successfull exit.
#ifdef __WXMSW__
		return NULL;
#else
		return EXIT_SUCCESS;
#endif
	}
	else
	{
		wxLogMessage(wxT("Fatal error, terminating."));
#ifdef __WXMSW__
		eventToMain(ID_fatalError);
		return NULL;
#else
		return EXIT_FAILURE;
#endif
	}
}
