/* $Id: kisstnc.h 445 2011-02-21 09:17:48Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009, 2010, 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef KISSTNC_H
#define KISSTNC_H

// own stuff.
#include "serial.h"
#include "cfg.h"
#include "packetlog.h"
#include "dupefilter.h"

// std stuff.
#include <string>
#include <vector>

// wx stuff.
#include <wx/string.h>
#include <wx/thread.h>



/// Implements Serial for a KISS TNC.
class KissTnc : public Serial
{
public:
	
	/// Simply passes all arguments to the constructor of Serial.
	KissTnc(Cfg::ConnectionCfg const& cfg, char const& conId, std::string const& owncall,
	        Packetlog* packetlog, DupeFilter* dupefilter, wxString const& lockDir);

	/// Destructor.
	virtual ~KissTnc();
	
	// This gets automatically called upon starting the thread.
	virtual ExitCode Entry();
	
private:

	/// Address of device is stored also here for error messaging reasons.
	wxString const device_;
	
	// Data storage
	
	/// Bytes received from device are stored first here.
	std::string inputBuffer_;
	
	/// Once a complete KISS frame is found from inputBuffer_, it is moved here.
	std::string kissFrame_;
	
	/// Once a packet is parsed from a frame, it's stored here.
	std::string tnc2Packet_;


	// KISS frame parsing functions.
	
	/// Look for KISS frame in incomingByteBuffer_.
	/**
	 * Once a complete frame is found, it's moved into incomingKissFrame_.
	 * \return True, when finds a complete KISS frame.
	 */
	bool findKissFrame(std::string& frame);
	
};

#endif // KISSTNC_H
