/* $Id: ax25port.h 451 2011-02-26 10:32:30Z oh2gve $
 *
 * Copyright 2009, 2010, 2011 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef AX25PORT_H
#define AX25PORT_H

// Compile contents only if AX.25 support is enabled.
#ifdef HAVE_LIBAX25


// own stuff.
#include "../rfcon.h"
#include "../cfg.h"
#include "../packetlog.h"
#include "../dupefilter.h"

// wx stuff.
#include <wx/string.h>



/// Implements Connection for AX25 interface.
class Ax25Iface : public RFCon
{
public:
	
	/// Constructor.
	Ax25Iface(Cfg::ConnectionCfg const& cfg, char const& conId, wxString const& owncall, Packetlog* packetlog, DupeFilter* dupefilter);
	
	/// Destructor.
	virtual ~Ax25Iface();

	// Entry point of our thread. The actual "main" function for this  thread.
	virtual void* Entry();

private:
	
	/// Receiving socket.
	int rxSocket_;
	/// Transmitting socket.
	int txSocket_;

	// State variables
	
	/// Tells Entry() how constructor did.
	bool initialized_;
	
	/// Name of the port we're supposed to use.
	wxString const port_;
	
	/// Interface name of the port, eg. ax0.
	wxString ifaceName_;
	
	/// Callsign of the port, eg. MYCALL.
	wxString portCall_;
	
	/// Our callsign.
	wxString const owncall_;
	
};


#endif // HAVE_LIBAX25

#endif // AX25PORT_H
