/* $Id: rfcon.cpp 448 2011-02-26 07:26:08Z oh2gve $
 *
 * Copyright 2007 Antti Seppänen
 * Copyright 2007, 2008, 2009, 2010, 2011 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/


// Build config.
#ifdef __WXMSW__
#	include "windows/config.h"
#endif
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

// Own stuff.
#include "rfcon.h"
#include "connection.h"
#include "cfg.h"
using Cfg::ConnectionCfg;
#include "packet.h"
#include "aprspacket.h"
#include "packetlog.h"
#include "dupefilter.h"
#include "filter.h"

// std stuff.
#include <string>
using std::string;



RFCon::RFCon(ConnectionCfg const& cfg, char const& conId, string const& owncall, Packetlog* packetlog, DupeFilter* dupefilter):
Connection(cfg.state, conId, packetlog, dupefilter),
owncall_(owncall), mypath_(),
packetlog_(packetlog), dupefilter_(dupefilter), filter_(cfg.filterElements), hold_(cfg.limitercfg.holdtime)
{
	// Convert path to std::string.
	for ( unsigned int i = 0; i < cfg.path.size(); i++ )
	{
		mypath_.push_back( static_cast<char const*>(cfg.path.at(i).ToAscii()) );
	}
}


RFCon::~RFCon()
{
}


// PROTECTED

void RFCon::input(AprsPacket* const& data)
{
	// Dummy check.
	if ( data == NULL )
	{
		return;
	}

	// Inform filter about this packet. It wants to know packets originating
	// from this network, so this is done before 3rd party detection.
	filter_.incomingPacket(data);
	
	// Try to get third-party packet from the packet.
	AprsPacket* actual = data;
	while ( AprsPacket* nextresult = actual->getCarriedThirdParty() )
	{
		// Remove temporary packet.
		if ( actual != data )
		{
			delete actual;
		}
		
		// Update actual.
		actual = nextresult;
	}
	
	// Drop copies of the actual packet from hold.
	hold_.inform(actual);
	
	// Upper layer needs packet as seen on RF, so the possible actual packet riding in it must be deleted.
	if ( actual != data )
	{
		delete actual;
	}
	
	// Push data to input buffer.
	Connection::inputPush(data);
}


AprsPacket* RFCon::output()
{
	// Check for packets from main.
	Packet* packet = Connection::outputPop();
	if ( packet != NULL )
	{
		// Check for own transmissions.
		if ( packet->getInputId() == '_' )
		{
			// No restriction measures for own packets.
		}
		// Check if this passes our filter.
		else if ( !filter_.outgoingPacket(packet->getAprsPacket()) )
		{
			// No, discard it.
			delete packet;
			packet = NULL;
		}
		else
		{
			// Put in hold.
			hold_.put(packet);
			packet = NULL;
		}
	}
	
	// If no packet, check for one from hold.
	if ( packet == NULL )
	{
		packet = hold_.get();
	}
	
	// Process packet.
	if ( packet != NULL )
	{
		// Set timetamp to current time.
		packet->updateTimestamp();
	
		// Create a copy to send.
		AprsPacket* retval = NULL;
		if ( packet->getInputId() == '_' )
		{
			// Own packet needs no modification.
			retval = new AprsPacket(*packet->getAprsPacket());
		}
		else
		{
			// Packets coming from other connections are converted to third party format.
			retval = new AprsPacket(owncall_, OWN_TARGET, mypath_, packet->getAprsPacket()->getAsThirdParty(owncall_, packet->getInputName()), true);
		}
		
		// Put original to log.
		packetlog_->write(packet);
		
		// Send packet out.
		return retval;
	}
	
	return NULL;
}
