/* $Id: serial.h 445 2011-02-21 09:17:48Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009, 2010, 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef SERIAL_H
#define SERIAL_H

// own stuff.
#include "rfcon.h"
#include "cfg.h"
#include "packetlog.h"
#include "dupefilter.h"
#include "serialport.h"

// std stuff.
#include <string>
#include <vector>

// wx stuff.
#include <wx/string.h>
#include <wx/thread.h>


/// Common functionality of serial port based connections.
/**
 * Handles serial port device. Reports errors on serial port operations
 * via wxLogMessage.
 */
class Serial : public RFCon
{
public:
	
	/// Constructor tries to open the device and calls connected().
	Serial(Cfg::ConnectionCfg const& cfg, char const& conId, std::string const& owncall,
	       Packetlog* packetlog, DupeFilter* dupefilter, wxString const& lockDir);

	/// Destructor closes the device.
	virtual ~Serial();

	// Implementation of the "main()" of each thread is forwarded to child classes.
	virtual ExitCode Entry() = 0;
	
protected:
	
	/// Read bytes from serial port.
	/**
	 * \param data Bytes read from the device or an empty string in case of timeout.
	 * \return True in case of bytes received or timeout, false if error.
	 */
	bool readDevice(std::string& data);
	
	/// Write given bytes to serial port.
	/**
	 * \return True if data was written successfully, false otherwise.
	 */
	bool writeToDevice(std::string const& data);
	
	/// Tells our children how the serial port is doing.
	bool portIsConnected() const;
	
private:
	
	/// File name of the serial port device.
	wxString deviceFile_;
	
	/// The serial port device itself.
	Serialport* device_;

};

#endif // SERIAL_H
